import { ObservableMap } from "./observable-map.js";
import { createContext, ContextEvent, } from "./context-protocol.js";
export function ProviderMixin(Class) {
    return class extends Class {
        #dataStore = new ObservableMap();
        connectedCallback() {
            super.connectedCallback?.();
            const contexts = "contexts" in this ? this.contexts : {};
            for (const [key, value] of Object.entries(contexts || {})) {
                this.#dataStore.set(key, value());
            }
            this.addEventListener("context-request", this.#handleContextRequest);
        }
        disconnectedCallback() {
            this.#dataStore = new ObservableMap();
            this.removeEventListener("context-request", this.#handleContextRequest);
        }
        updateContext(name, value) {
            this.#dataStore.set(name, value);
        }
        // We listen for a bubbled context request event and provide the event with the context requested.
        #handleContextRequest(event) {
            const { name, initialValue } = event.context;
            const subscribe = event.subscribe;
            if (initialValue) {
                this.#dataStore.set(name, initialValue);
            }
            const data = this.#dataStore.get(name);
            if (data) {
                event.stopPropagation();
                let unsubscribe = () => undefined;
                if (subscribe) {
                    unsubscribe = () => {
                        data.subscribers.delete(event.callback);
                    };
                    data.subscribers.add(event.callback);
                }
                event.callback(data.value, unsubscribe);
            }
        }
    };
}
export function ConsumerMixin(Class) {
    return class extends Class {
        #unsubscribes = [];
        getContext(contextName) {
            let result;
            this.dispatchEvent(new ContextEvent(createContext(contextName), (data) => {
                result = data;
            }));
            return result;
        }
        connectedCallback() {
            super.connectedCallback?.();
            const contexts = "contexts" in this ? this.contexts : {};
            for (const [contextName, callback] of Object.entries(contexts || {})) {
                const context = createContext(contextName);
                // We dispatch a event with that context. The event will bubble up the tree until it
                // reaches a component that is able to provide that value to us.
                // The event has a callback for the the value.
                this.dispatchEvent(new ContextEvent(context, (data, unsubscribe) => {
                    callback(data);
                    if (unsubscribe) {
                        this.#unsubscribes.push(unsubscribe);
                    }
                }, 
                // Always subscribe. Consumers can ignore updates if they'd like.
                true));
            }
        }
        // Unsubscribe from all callbacks when disconnecting
        disconnectedCallback() {
            for (const unsubscribe of this.#unsubscribes) {
                unsubscribe?.();
            }
            // Empty out the array in case this element is still stored in memory but just not connected
            // to the DOM.
            this.#unsubscribes = [];
        }
    };
}
